# -*- coding: UTF-8 -*-
#
#  FstabDialogs.py : Provide dialogs for FstabHandler
#  Copyright (C) 2007 Mertens Florent <flomertens@gmail.com>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import pygtk
pygtk.require("2.0")
import gtk
import gtk.glade
from gettext import gettext as _

from Fstabconfig import *


def warning_dial(title, text, parent) :
    ''' Show a warning message '''
    
    dialog = gtk.MessageDialog(parent = parent,
                               flags = gtk.DIALOG_MODAL,
                               type = gtk.MESSAGE_WARNING,
                               buttons = gtk.BUTTONS_OK,
                               message_format = None)
                               
    dialog.set_markup("<big><b>" + title + "</b></big>")
    dialog.format_secondary_markup(text)
    dialog.run()
    dialog.destroy()
    
def question_dial(title, text, parent) :
    ''' Ask a question '''

    dialog = gtk.MessageDialog(parent = parent,
                               flags = gtk.DIALOG_MODAL,
                               type = gtk.MESSAGE_QUESTION,
                               buttons = gtk.BUTTONS_YES_NO,
                               message_format = None)
                               
    dialog.set_markup("<big><b>" + title + "</b></big>")
    dialog.format_secondary_markup(text)
    res = dialog.run()
    dialog.destroy()
    return res
    
def error_dial_message(title, text, message, parent) :

    gladexml =  gtk.glade.XML(GLADEFILE, "dialog_error_message")
    dialog = gladexml.get_widget("dialog_error_message")
    dialog.set_title("")
    if parent : dialog.set_transient_for(parent)
    title_label = gladexml.get_widget("title_label")
    title_label.set_label("<big><b>" + title + "</b></big>")
    text_label = gladexml.get_widget("text_label")
    text_label.set_label(text)
    textview = gladexml.get_widget("textview")
    buf = gtk.TextBuffer()
    buf.set_text(message)
    l = message.split("\n")
    if len(l) < 10 :
        height = int(225 + 10 * len(l))
    else :
        height = 375
    if len(l[0]) < 50 :
        width = int(425 + 2 * len(l[0]))
    else :
        width = 550
        height += 12
    dialog.set_size_request(width, height)
    textview.set_buffer(buf)
    ret = dialog.run()
    dialog.destroy()
    
def warning_dial_list(title, text_top, entry_list, text_down,\
         action, parent = None, sensitive = True, default = True) :

    gladexml =  gtk.glade.XML(GLADEFILE, "dialog_warning_list")
    dialog = gladexml.get_widget("dialog_warning_list")
    dialog.set_title("")
    if parent : dialog.set_transient_for(parent)
    title_label = gladexml.get_widget("title_label")
    title_label.set_label("<big><b>" + title + "</b></big>")
    text_top_label = gladexml.get_widget("text_top_label")
    text_top_label.set_label(text_top)
    text_down_label = gladexml.get_widget("text_down_label")
    text_down_label.set_label(text_down)
    action_button = gladexml.get_widget("action_button")
    action_button.set_label(action)
    if len(entry_list) < 6 :
        height = int(250 + 15 * len(entry_list))
    else :
        height = 375
    if len(entry_list[0]) < 75 :
        width = int(400 + 2 * len(entry_list[0]))
    else :
        width = 550
        height += 15
    dialog.set_default_size(width, height)
    
    def on_toggled(renderer, path_string) :
    
        tree_store[path_string][0] = not tree_store[path_string][0]
        
    treeview = gladexml.get_widget("treeview")
    renderer= gtk.CellRendererToggle()
    renderer.set_property("activatable", sensitive)
    renderer.set_property("sensitive", sensitive)
    renderer.connect("toggled", on_toggled)
    column = gtk.TreeViewColumn("", renderer, active=0)
    treeview.append_column(column)
    renderer = gtk.CellRendererText()
    column = gtk.TreeViewColumn("", renderer, text=1)
    treeview.append_column(column)
    tree_store = gtk.ListStore(bool, str)
    treeview.set_model(tree_store)
    for entry in entry_list :
        tree_store.append((default, entry))
    ret = dialog.run()
    if ret == gtk.RESPONSE_DELETE_EVENT :
        ret = gtk.RESPONSE_CANCEL
    elif not ret == gtk.RESPONSE_CANCEL :
        select = []
        unselect = []
        iter = tree_store.get_iter_first()
        for i in range(len(entry_list)) :
            if tree_store.get_value(iter, 0) :
                select.append(i)
            else :
                unselect.append(i)
            iter = tree_store.iter_next(iter)
        ret = (select, unselect)
    dialog.destroy()
    return ret

if __name__ == "__main__" :

    dial = warning_dial_list(
        "Removing unknow devices from\nthe configuration file ?", \
        "I can not find any existant block devices\n"
        "corresponding to the following devices :", \
        [ "O" * 26] *1, \
        "It is adviced to remove them in order to avoid failed mount\n"
        "at startup. Select the ones you want to remove.", \
        "Remove selected")

