# -*- coding: UTF-8 -*-
#
#  Utility.py : Various utility for Fstab
#  Copyright (C) 2007 Mertens Florent <flomertens@gmail.com>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import os
import sys
import time
from commands import *
from gettext import gettext as _

from Fstabconfig import *
from Fstab import *
from FstabDialogs import *
import FstabData

def get_used_file(path) :

    cmd = "%s |grep -F %s" %(LSOF, escape_special(path))
    lines = getoutput(cmd).split("\n")
    result = []
    for line in lines :
        used = path + line.split(path)[-1].strip()
        if ( used == path or not used.find(path + "/") == -1 ) and len(line.split()) > 0 :
            result.append(line.split()[0] + " -> " + used)
    return result
  
def get_dmesg_output() :

    lines = getoutput(DMESG + " | tail").split("\n")
    try :
        return "\n".join(["%s" % k.split("]")[1] for k in lines ])
    except :
        return "\n".join(lines)

def check_unknow(fstab, parent) :
    
    unknow_block = []
    for entry in fstab.other :
        if re.search("(^UUID=|^LABEL=|^/dev/)", entry["FSTAB_NAME"]) \
            and entry["FSTAB_TYPE"] not in FstabData.ignore_fs \
            and re.search("^/", entry["FSTAB_PATH"]) :
            unknow_block.append(entry)
    if unknow_block :
        dial = warning_dial_list(
            _("Removing unknown devices from\nthe configuration file?"), \
            _("I can not find any existing block devices\n"
            "corresponding to the following devices:"), \
            [ "%s on %s" % (k["FSTAB_NAME"], k["FSTAB_PATH"]) for k in unknow_block ], \
            _("It is advisable to remove them in order to avoid failed mount\n"
            "at start-up. Select the ones you want to remove."), \
            _("Remove selected"), parent)
        if not dial ==  gtk.RESPONSE_CANCEL :
            for i in dial[0] :
                fstab.other.remove(unknow_block[i])
            fstab.apply()      

def check_duplicate(fstab, parent) :

    checked = []
    for device in fstab.list() :
        index = fstab.search(device)
        index.reverse()
        for i in index[:] :
            if fstab.count(fstab[i]) > 1 :
                logging.warning("Removing exact duplicate entry %s" % fstab[i]["DEVICE"])
                fstab.remove(i)
                index.remove(i)
        index = fstab.search(device)
        index.reverse()
        if len(index) > 1 and not device in checked :
            entries = [ "%s on %s" % (fstab[k]["FSTAB_NAME"], \
                fstab[k]["FSTAB_PATH"]) for k in index ]
            dial = warning_dial_list(
            _("Duplicate entry detected"), \
            _("The following entries of your configuration file are duplicates:"), \
            entries, \
            _("This program can not manage for the moment duplicate entries\n"
            "You should select the one you want, and the others will be commented.\n"
            "They will be kept in the configuration file, but will be disabled."), \
            _("Keep selected"), parent, default = False)
            if not dial ==  gtk.RESPONSE_CANCEL :
                for j in dial[1] :
                    i = index[j]
                    logging.debug("Commenting %s" % entries[j])
                    entry = ["#%s" % fstab[i]["FSTAB_NAME"]]
                    entry.extend([ fstab[i][u] for u in FstabData.categorie[1:] ])
                    fstab.remove(i)
                    fstab.comment.append(EntryBase(entry))
                fstab.apply()
        checked.append(device)

def change_tracker(x, y) :
    ''' Track changes between two version of a MntFile '''
        
    (log, to_mount, to_umount) = ([], [], [])
    if not x == y :
        for i in range(len(y)) :
        
            iy = y[i]
            if not x.search(iy["DEVICE"]) :
                log.append("Adding " + iy["DEVICE"])
                if not device_is_mounted(iy["DEVICE"]) == iy["FSTAB_PATH"] :
                    to_mount.append(iy)
                continue
                
            ix = x[iy["DEVICE"]]
            if not iy["FSTAB_PATH"] == ix["FSTAB_PATH"] :
                log.append(iy["DEVICE"] + ": Changing path :\n " + \
                    ix["FSTAB_PATH"] + " -> " + iy["FSTAB_PATH"])
                if not iy["FSTAB_PATH"] == device_is_mounted(iy["DEVICE"]) and ix.get_is_mounted() :
                    to_mount.append(iy)

            if not iy["FSTAB_TYPE"] == ix["FSTAB_TYPE"] :
                log.append(iy["DEVICE"] + ": Changing fs :\n " + \
                    ix["FSTAB_TYPE"] + " -> " + iy["FSTAB_TYPE"])
                if not iy in to_mount and ix.get_is_mounted() : 
                    to_mount.append(iy)  
                continue

            if not iy["FSTAB_OPTION"] == ix["FSTAB_OPTION"] :
                log.append(iy["DEVICE"] + ": Changing options :\n " + \
                    ix["FSTAB_OPTION"] + " -> " + iy["FSTAB_OPTION"])
                if not iy in to_mount and ix.get_is_mounted() :
                    for opt in ix.listopt() :
                        if opt not in iy.listopt()  \
                                and opt not in FstabData.dont_need_remount :
                            to_mount.append(iy)
                            break
                if not iy in to_mount and ix.get_is_mounted() :
                    for opt in iy.listopt() :
                        if opt not in ix.listopt() \
                                and opt not in FstabData.dont_need_remount :
                            to_mount.append(iy)
                            break
                
            if not iy["FSTAB_PASO"] == ix["FSTAB_PASO"] :
                log.append(iy["DEVICE"] + ": Changing check at boot :\n " + \
                    ix["FSTAB_PASO"] + " -> " + iy["FSTAB_PASO"])
                    
        for i in range(len(x)) :
            ix = x[i]
            if not y.search(ix["DEVICE"]) :
                log.append("Removing " + ix["DEVICE"])
                to_umount.append(ix)
   
    return (log, to_mount, to_umount)

    
