# -*- coding: UTF-8 -*-
#
#  Mounter.py : A GTK mounter for FstabHandler
#  Copyright (C) 2007 Mertens Florent <flomertens@gmail.com>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import os
import time
from xml.sax.saxutils import escape as escape_mkup

import pygtk
pygtk.require("2.0")
import gtk
import gobject
import gtk.glade
from SimpleGladeApp import SimpleGladeApp
from gettext import gettext as _

from Fstabconfig import *
from FstabDialogs import *
from Fstab import *
from FstabUtility import *


class Mounter(SimpleGladeApp) :
    ''' Show a progress bar while mounting a list of entry '''
    
    def __init__(self, parent = None) :
        
        SimpleGladeApp.__init__(self, GLADEFILE, "window_mounting_progress", domain = PACKAGE)
        self.window_mounting_progress.set_title("")
        self.parent = parent
        if self.parent :
            self.window_mounting_progress.set_transient_for(self.parent)
        
    def mount(self, devices, action = "mount") :
        ''' Generic mounter. Default is to mount devices (in list of entry).
            To only umount the devices, set action to umount '''

        if not devices :
            return (0, 0)
        self.devices = devices
        self.action = action
        (self.res_umount, self.res_mount) = (0, 0)
        if self.action == "mount" : 
            self.label = _("Mounting partitions")
        else : 
            self.label = _("Unmounting partitions")
        self.title.set_label("<big><b>%s</b></big>" % self.label)
        self.progressbar.set_fraction(0.0)
        self.progress_label.set_label("<i>%s</i>" % self.label)
        self.window_mounting_progress.show_now()
        while gtk.events_pending() :
            gtk.main_iteration()
        self.mounter()
        time.sleep(0.05)
        self.window_mounting_progress.hide()
        return (self.res_umount, self.res_mount)
        
    def update(self, text, step, total) :

        if step > 0:
            self.progressbar.set_fraction(step/float(total))
        if text != "":
            self.progress_label.set_label(text)
        while gtk.events_pending() :
            gtk.main_iteration()
        
    def restore_gui(self) :
    
        while gtk.events_pending() :
            gtk.main_iteration()
        time.sleep(0.005)
        while gtk.events_pending() :
            gtk.main_iteration()
    
    def mounter(self) :

        total = len(self.devices)
        for i in range(total) :
            text = escape_mkup(self.devices[i]["FSTAB_PATH"])
            if self.action == "mount" :
                label = _("Mounting %s" % text) 
            else : 
                label = _("Unmounting %s" % text) 
            self.update("<i>%s</i>" % label, i, total)
            if device_is_mounted(self.devices[i]["DEVICE"]) :
                self.res_umount = self.umount_device(self.devices[i])
                if self.res_umount or not self.action == "mount" :
                    continue
                time.sleep(0.1)
            if not self.action == "mount" :
                continue
            self.res_mount = self.mount_device(self.devices[i])
        self.update("", total, total)
        
    def umount_device(self, entry) :
    
        count = 0
        while not count == -1  :
            count += 1
            ret = entry.umount()
            if ret[0] :
                self.used_file = get_used_file(device_is_mounted(entry["DEVICE"]))
                if self.used_file :
                    dial_ret = warning_dial_list(_("Unmounting failed"), \
                        _("Unmounting <i>%s</i> failed, because\n" \
                        "it is currently used by the following applications:") \
                        % entry["DEVICE"], self.used_file, \
                    _("Close all applications that could use it and retry to unmount."), \
                    _("Retry unmount"), self.window_mounting_progress, sensitive=False)
                    self.restore_gui()
                    if dial_ret == gtk.RESPONSE_CANCEL :
                        count = -1
                else :
                    if count <= 3 :
                        logging.debug("Attempt %i failled. Retry in %i s..." % (count, 2*count))
                        gobject.timeout_add(2000*count, gtk.main_quit)
                        gtk.main()
                        continue
                    error_dial_message(_("Unmounting failed"), \
                        _("Unmounting <i>%s</i> failed because of the following error:") \
                        % entry["DEVICE"], ret[1].strip(), self.window_mounting_progress)
                    self.restore_gui()
                    count = -1
            else :
                count = -1
        return ret[0]
        
    def mount_device(self, entry) :
    
        ret = entry.mount()
        if ret[0] :
            text = _("Mounting <i>%s</i> failed because of the following error:") % entry["DEVICE"]
            msg = ret[1].strip()
            if not ret[1].find("dmesg") == -1 :
                # This string follow "Mounting ... failled because ... :" but only appear in certain case
                text += "\n%s" % _("(followed by the output of dmesg | tail)")
                msg += "\n\ndmesg | tail:\n" + get_dmesg_output()
            error_dial_message(_("Mounting failed"), text, msg, self.window_mounting_progress)
            self.restore_gui()
        return ret[0]
        
if __name__ == "__main__" :

    fstab = MntFile(FSTAB)
    mounter = Mounter()
    mounter.mount([fstab["/dev/hda1"]])
    mounter.mount([fstab["/dev/hda1"]])
    mounter.mount([fstab["/dev/hda1"]])
    mounter.mount([fstab["/dev/hda1"]])

